/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

import state.*;
import state.events.*;
import controller.*;
import agent.*;
import agent.autonomous.*;


/* ====================================================================== */
/** This class is for starting the simulator with a Scenario.  By starting
 *  iSIM with an XML scenario you can have a group of agents, already
 *  created and positioned when the system starts.
 *
 *  COMING SOON... With the scenario startup you will get a state gui, that
 *  lets you run through a scenario.  A scenario is a list of events, where
 *  events are various types of messages that get sent from agent to agent.
 *  This will allow a user to write various scripts for testing wireless
 *  applications.
 *
 */
public class IScenario{


	/* ====================================================================== */
  	/** The main method that creates an instance of Controller which will begin
  	 *  the simulation.
  	 */
  	public static void main (String argv[]){

		String filename = "scenario.xml";
		if(argv.length > 0){
			if( argv[0].indexOf( ".xml") > 0 ){
				filename = argv[0];
			}
		}

		Controller controller = new Controller();

		ScenarioParser sp = new ScenarioParser(filename, controller);

		controller.setUsers( sp.getUsers() );
		controller.setPoints( sp.getPoints() );
		controller.setCurrentAgent( (User)(sp.getUsers().elementAt(0)) );

		System.out.println( "             ---------  " + sp.getUsers().size());

		for(int i=0; i < sp.getUsers().size(); i++){
			controller.updateAgentPosition( (Agent)(sp.getUsers().elementAt(i)) );
			//create the client side.
			if( sp.getEvents().size() > 0 ){
				new StateClient( ((User)(sp.getUsers().elementAt(i))).getPort(), i);
			}
		}

		for(int i=0; i < sp.getPoints().size(); i++){
			controller.updateAgentPosition( (Agent)(sp.getPoints().elementAt(i)) );
		}

		controller.fireControllerChange();

		if( sp.getEvents().size() > 0 ){
			StateMachine sm = new StateMachine(controller, sp.getEvents() );
			StateWindow sw = new StateWindow(controller.getFrame(), sm);
		}

  	} //method: main


} //class: IScenario