/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import gui.*;
import agent.*;
import util.*;
import controller.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;




/* ====================================================================== */
/** This class is the Access Points tab in the TrayDialog.  It displays a list
 *  of all access points in the simulation, and enables adding and removing of
 *  access points.
 *
 */
public class PnlTabAccessPoints extends JPanel implements ControllerListener {


	/** The simulation controller that keeps all data. */
	private Controller controller;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

    /** Add user button. */
    private JButton btnAdd = new JButton(" Add ");

    /** REmove user button. */
    private JButton btnRemove = new JButton("Remove");

    /** Scrollpane for JTable */
    private JScrollPane spnUsers = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableUsers = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "Name", "Latitude", "Longitude", "Range"};

	/** A popup dialog for adding new user. */
	private JDialog addDialog;

    /** Field for uer range. */
    private JTextField range_field;






    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public PnlTabAccessPoints( Controller controller ) {

		this.controller = controller;
		controller.addControllerListener(this);

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }


    } //Constructor



    // ======================================================================
    /** Initializes this JPanel with Components.  Adds the table, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

      	bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        this.setBorder(TTBorder.getBorder("Access Points"));
        this.setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

        this.add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(btnAdd,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 0, 0), 0, 0));

        btnPanel.add(btnRemove,              new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 0, 0), 0, 0));


        this.add(spnUsers,  BorderLayout.CENTER);


		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableUsers = new JTable(dtm);

        spnUsers.getViewport().add(tableUsers, 0);



		btnAdd.setMnemonic(KeyEvent.VK_A);
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdAdd_actionPerformed();
              }
        });
		btnRemove.setMnemonic(KeyEvent.VK_R);
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btdRemove_actionPerformed();
            }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));

    } //method: init



    // ========================================================================
    /** This method is called to add a new user to the simulation.
     *
     */
    public void btdAdd_actionPerformed() {


		//Setup a current/new JDialog.
		addDialog = new JDialog(controller.getFrame());
		addDialog.setTitle("Add Access Point");

		//Create a button and the labels.
		JLabel lbl = new JLabel("Enter access point range below.");

		JButton btnAdd = new JButton( " ADD ");
		JButton btnCancel = new JButton( "CANCEL");


		JLabel rangelbl = new JLabel(" Wireless Range: "  );
		range_field = new JTextField(15);
		range_field.setText("100");

		//Create panels for hold the Components
		JPanel rangepanel = new JPanel(new GridBagLayout() );

		JPanel btnpanel = new JPanel(new GridBagLayout());
		JPanel lblpanel = new JPanel(new BorderLayout());
		lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 15, 15) );


        rangepanel.add(rangelbl, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        rangepanel.add(range_field, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));




        btnpanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(2, 2, 2, 2), 0, 0));

        btnpanel.add(btnCancel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 0, 0), 0, 0));


		//Add components to the panels, then add panels to the JDialog.


		lblpanel.add(lbl, BorderLayout.NORTH);
		lblpanel.add(rangepanel, BorderLayout.EAST);

		addDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
		addDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);


        range_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });

		//Make a button listener to close the dialog.
		btnAdd.setMnemonic(KeyEvent.VK_A);
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btdDialogAdd_actionPerformed();
			}
		});

		btnCancel.setMnemonic(KeyEvent.VK_C);
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				addDialog.dispose();
			}
		});

		//Pack it all together
		addDialog.pack();

		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (addDialog.getSize().width /2 );
		int y = ( screen.height / 2) - (addDialog.getSize().height /2 );
		addDialog.setLocation( x, y);

		//Now show this dialog.
		addDialog.setVisible(true);

    } //method:




    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btdRemove_actionPerformed() {

		TableModel tm = tableUsers.getModel();

		int row = tableUsers.getSelectedRow();

		String name = (String)tm.getValueAt(row, 0);

		Vector points = controller.getPoints();
		Agent use;

		for (int i=0; i < points.size(); i++) {
			use = (Agent)points.get(i);
			if(use.getName().equals(name) ){
				controller.removeAccessPoint((AccessPoint)use);
			}
		}

    } //method: btdRemove_actionPerformed


    // ========================================================================
    /** This method is
     *
     */
    public void btdDialogAdd_actionPerformed() {

		String n = range_field.getText();


		if( n.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a range.",
				"Illegal Range Number",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		try{
			int range = Integer.parseInt(n);

			if( range < 1 ){
				JOptionPane.showMessageDialog(addDialog,
					"Please enter a range larger than 0.",
					"Illegal Port Number",
					JOptionPane.INFORMATION_MESSAGE );
				addDialog.requestFocus();
				return;
			}



			controller.addAccessPoint( range );


		}catch(Exception ex){

			ex.printStackTrace();

			JOptionPane.showMessageDialog(addDialog,
				"Please enter only numbers in the fields.",
				"Bad Input",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		addDialog.dispose();
	}

     /* ======================================================================== */
    /** Called when the enter key is pressed in the text fields. This method is
     *  called by the KeyListener of the textfields.
     */
    private void enterKeyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        if (key == KeyEvent.VK_ENTER) {
            btdDialogAdd_actionPerformed();
        }

    }



    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the JTable, so as to keep it up to date.
     *
     */
	public void refresh(){

		Vector points = controller.getPoints();
		Position pos;
		Agent use;

		String[][] rowData = new String[points.size()][colNames.length]; // The data array

		for (int i=0; i < points.size(); i++) {
			use = (Agent)points.get(i);
			pos = use.getPosition();

	  		rowData[i][0] = use.getName();
	  		rowData[i][1] = ""+pos.getLatitude();
	  		rowData[i][2] = ""+pos.getLongitude();
	  		rowData[i][3] = ""+use.getRange();

		}

        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableUsers = new JTable(dtm);

        spnUsers.getViewport().add(tableUsers, 0);

	} //method: refresh



} //class: PnlTabAccessPoints