/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import controller.*;
import gui.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;




/* ====================================================================== */
/** This class is a simple JPanel whose sole purpose is to hold a set
 *  Tabbed Panes.
 *
 */
public class PnlTabs extends JPanel {

	/** The simulation controller. */
    private Controller controller;

  	/** The tab pane that contains all tabs. */
  	private JTabbedPane tabMain = new JTabbedPane();

	/** People tab. */
    private PnlTabPeople pnlPeople;

    /** Access Point tab. */
    private PnlTabAccessPoints pnlAccessPoints;

    /** Tool tab. */
    private PnlTabTools pnlTools;



    // ======================================================================
    /** Constructor instantiates the individual tabs and calls init.
     *
     *  @param  controller   The simulation controller.
     *
     */
    public PnlTabs(Controller controller) {

        this.controller = controller;

		// Setup the tabs
        pnlPeople = new PnlTabPeople( controller );
        pnlAccessPoints = new PnlTabAccessPoints( controller );
        pnlTools = new PnlTabTools( controller );


        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    } //Constructor



    // ======================================================================
    /** Initializes this JPanel by setting the border, and adding the tabs
     *  to the tabbed pane.
     *
     */
    private void init() throws Exception {

        this.setLayout(new BorderLayout());

        this.add(tabMain, BorderLayout.CENTER);

        tabMain.add(pnlPeople,  "People");
        tabMain.add(pnlAccessPoints,  "Access Points");
        tabMain.add(new PnlTabRobots( controller ),  "Robots");
        tabMain.add(new PnlTabRecorder( controller ),  "Recorder");
        tabMain.add(pnlTools,  "Tools");


    } //method: init


} //class: PnlTabs